/****************************************************************************
 Module
   FollowerService.c
 ****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
 */
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_DeferRecall.h"
#include "FollowerService.h"
#include "MotorSM.h"
#include "PIC32_SPI_HAL.h"
#include "sys/attribs.h"

/*----------------------------- Module Defines ----------------------------*/

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this service.They should be functions
   relevant to the behavior of this service
 */

/*---------------------------- Module Variables ---------------------------*/
// with the introduction of Gen2, we need a module level Priority variable
static uint8_t MyPriority;

static uint8_t ByteReceived;
static FollowerState_t CurrentState;

static ES_Event_t DeferralQueue[5 + 1];

/*------------------------------ Module Code ------------------------------*/

/****************************************************************************
 Function
     InitFollowerService

 Parameters
     uint8_t : the priority of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, and does any
     other required initialization for this service
 Notes

 Author
     J. Edward Carryer, 01/16/12, 10:00
 ****************************************************************************/
bool InitFollowerService(uint8_t Priority) {
    ES_Event_t ThisEvent;

    MyPriority = Priority;
    // initialize deferral queue for testing Deferal function
    ES_InitDeferralQueueWith(DeferralQueue, ARRAY_SIZE(DeferralQueue));

    InitSPI();
    CurrentState = InitFollower;

    __builtin_enable_interrupts();

    // post the initial transition event
    ThisEvent.EventType = ES_INIT;
    if (ES_PostToService(MyPriority, ThisEvent) == true) {
        return true;
    } else {
        return false;
    }
}

bool PostFollowerService(ES_Event_t ThisEvent) {
    return ES_PostToService(MyPriority, ThisEvent);
}

ES_Event_t RunFollowerService(ES_Event_t ThisEvent) {
    ES_Event_t ReturnEvent;
    ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
    
//    if(ThisEvent.EventType == ES_INIT) {
//        ES_Timer_InitTimer(SERVICE0_TIMER, 378);
//    }
//    
//    if (ThisEvent.EventType == ES_TIMEOUT) {
//        ES_Timer_InitTimer(SERVICE0_TIMER, 378);
//        SPIOperate_SPI1_Send8(0x04);
//    }
    if (ThisEvent.EventType == HW_TARGETREACHED) {
        SPIOperate_SPI1_Send8(0x04);
        printf("SENDING TARGET REACHED\r\n");
    }
    
    if (ThisEvent.EventType == HW_SPI_RX) {
        switch(ThisEvent.EventParam) {
            case 0x01:
            {
                printf("\rBeacon\r\n"); 
                ES_Event_t Beacon = {HW_BEACON, 0};
                PostMotor(Beacon);
            }
            break;
            case 0x02:
            {
                printf("\rDunk\r\n");
                ES_Event_t Dunk = {HW_DUNKMODE, 0};
                PostMotor(Dunk);
            }
            break;
            case 0x03:
            {
                printf("\rLoad\r\n");
                ES_Event_t Load = {HW_LOADMODE, 0};
                PostMotor(Load);
            }
            break;
            case 0x04:
            {
                printf("\rWARNING: SHOULD NOT RECEIVED 'Target (0x03)' BUT RECEIVED\r\n");
            }
            break;
            case 0x05:
            {
                printf("\rCompleted Dunk\r\n");
                ES_Event_t Complete = {HW_DUNKCOMPLETE, 0};
                PostMotor(Complete);
            }
            break;
            case 0x08:
            {
                printf("\rMisaligned\r\n");
                ES_Event_t Misalign = {HW_MISALIGNED, 0};
                PostMotor(Misalign);
            }
        }
    }

    return ReturnEvent;
}

/***************************************************************************
 private functions
 ***************************************************************************/

void __ISR(_SPI1_VECTOR, IPL5SOFT) _HW_SPI1Handler(void) {

    __builtin_disable_interrupts();
    do {
        IFS1CLR = _IFS1_SPI1RXIF_MASK;
        ByteReceived = SPI1BUF; // read new command
        
        ES_Event_t ReturnEvent;
        ReturnEvent.EventType = HW_SPI_RX;
        ReturnEvent.EventParam = ByteReceived;
        PostFollowerService(ReturnEvent);
    } while (!SPI1STATbits.SPIRBE);

    IFS1CLR = _IFS1_SPI1RXIF_MASK;

    __builtin_enable_interrupts();
}


/*------------------------------- Footnotes -------------------------------*/
/*------------------------------ End of file ------------------------------*/