/****************************************************************************
 Module
   MotorSM.c

 Revision
   1.0.1

 Description
   This is a template file for implementing flat state machines under the
   Gen2 Events and Services Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
 ****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
 */
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_Timers.h"

//include self
#include "MotorSM.h"
#include "UltrasonicService.h"
#include "SwitchChecker.h"
#include "FollowerService.h"

//to make ISRs
#include <xc.h>
#include <sys/attribs.h>

//for debugging, print onto the screen
#include "terminal.h"
#include "dbprintf.h"

/*----------------------------- Module Defines ----------------------------*/

//if set to 0, then Timer1 will NOT update OCs
//and open loop control will be used
#define CLOSEDLOOP  0


//define the pulse width (fall and rise time) with TIMER 3

//define PWM length in ticks
//THIS DOES NOT CONTROL THE TM3 INIT FUNCTION, that has to be chnged manually
//based on the TCKPS bit in T3CON
#define PWM_PRESCALAR 2

//desired pulse width frequency
#define KHERTZ 5

//define the pulse width to write to TM3 to control PWM frequency
#define PULSEWIDTH (((20000 / PWM_PRESCALAR) / KHERTZ) - 1) //at 20 Mhz, 20,000 ticks pass for 1 millisecond (1 kHz))

//if want to bypass frequency calc, setting OC period through PR
//#define PULSEWIDTH 65500

//to have the motors run in the SAME direction (forward) 
//OC1 = OC4 = 0
//OC2 & OC3 > 0)

//define the initial duty cycle OC1 Motor1 1/2, CW forward
//for finding tau, start with a PW of 1ms and DC of 50%
#define DC_INITIAL (PULSEWIDTH * 0)

//define the initial duty cycle OC2 Motor1 2/2, CW forward
//for finding tau, start with a PW of 0
#define DC_INITIAL_2 (PULSEWIDTH * 0)

//define the initial duty cycle OC2 Motor2 1/2, CW backward
//for finding tau, start with a PW of 0
#define DC_INITIAL_3 (PULSEWIDTH * 0)

//define the initial duty cycle OC2 Motor2 2/2, CW backward
//for finding tau, start with a PW of 0
#define DC_INITIAL_4 (PULSEWIDTH * 0)

//open loop speeds
#define DC_TURN 0.5
#define DC_FULLSPEED 1.6
#define DC_HALFSPEED DC_FULLSPEED / 2
#define MOTOR2_TO_MOTOR1_RATIO 0.95 // higher values bias left

//timer durations
#define RELOAD_FWD_DURATION 2000

//closed loop speeds (RPM)
#define RPM_TURN  7
#define RPM_SPEED 15
#define RPM_MAX_POSSIBLE 50
/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
 */

//sets the Timer3 frequency based on PWM_PRESCALAR, the PWM frequency based on
//PULSEWIDTH, and the initial duty cycle based on DC_INITIAL
//then RESTARTS BOTH TM3 AND OC1
static void TM3OCInit();

//sets Timer2 frequency consider encoder AND ultrasound sensor needs
//initialized IC1 for Motor 1 encoder and IC2 for Motor 2 encoder
static void TM2ICInit();

//sets Timer1 to run closed loop control in interrupt
static void TM1Init();

//Timer4 to check for stalling in M1 due to encoder not reading 0
static void TM4InitOnly();
static void TM4Start();

//Timer5 to check for stalling in M2 due to encoder not reading 0
static void TM5InitOnly();
static void TM5Start();

//for setting OpenLoop values to Motors 1 & 2
static void ForwardM1(float requestedDC);
static void ReverseM1(float requestedDC);
static void ForwardM2(float requestedDC);
static void ReverseM2(float requestedDC);

static void Forward_FullSpeed();
static void Reverse_FullSpeed();
static void Forward_HalfSpeed();
static void Reverse_HalfSpeed();
static void CW();
static void CCW();
static void Stop();

//for setting ClosedLoop values to Motor 1 & 2
static void CL_Reverse(uint8_t requestedRPM);
static void CL_Forward(uint8_t requestedRPM);
static void CL_CW(uint8_t requestedRPM);
static void CL_CCW(uint8_t requestedRPM);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static TemplateState_t CurrentState;

// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;

volatile FeatureWidthTime TimeofLastRise, TimeofLastFall, KeepTime;
volatile FeatureWidthTime2 TimeofLastRise2, TimeofLastFall2, KeepTime2;

//calculating RPM from encoder 1 (Motor1)
volatile static uint16_t WheelRPM = 0;

//calculating RPM from encoder 1 (Motor2)
volatile static uint16_t WheelRPM2 = 0;

//for use in CL control calc (Motor 1)
volatile static uint16_t DesiredVel = 0;
volatile static int16_t ErrorSum = 0;
volatile static int16_t ErrorLast = 0;

//for use in CL control calc (Motor 2)
volatile static uint16_t DesiredVel2 = 0;
volatile static int16_t ErrorSum2 = 0;
volatile static int16_t ErrorLast2 = 0;

//for use in CL control choosing reverse or forwards need to choose OC
//to write to , Forward is 1 Reverse is 0

//to have the motors forward 
// Motor 1 OC1, OC2 > 0
//Motor 2 OC4 = 0, OC3 > 0

volatile static uint16_t Direction = 1;
volatile static uint16_t Direction2 = 1;

//to prevent stall-outs, false if IT IS timed out

static bool StallOutTimer = false;
static bool StallOutTimer2 = false;

static bool GoingBack = false;


/*------------------------------ Module Code ------------------------------*/

/****************************************************************************
 Function
     InitTemplateFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
 ****************************************************************************/
bool InitMotor(uint8_t Priority) {
    ES_Event_t ThisEvent;

    MyPriority = Priority;
    // put us into the start of the Game
    CurrentState = GameStart;

    //initialize and start the TM3 and OCs
    TM3OCInit();

    //initialize and start the TM2 and ICs
    TM2ICInit();

    //initialize and start TM1 for CL control
    TM1Init();

    //initialize and start TM4 for stall control
    TM4InitOnly();
    TM4Start();

    //initialize and start TM4 for stall control
    TM5InitOnly();
    TM5Start();
    
    //Initialize Front Switch
    SwitchInit();
    
    GoingBack = false;
    
    // post the initial transition event
    ThisEvent.EventType = ES_INIT;
    if (ES_PostToService(MyPriority, ThisEvent) == true) {
        return true;
    } else {
        return false;
    }
}

/****************************************************************************
 Function
     PostTemplateFSM

 Parameters
     EF_Event_t ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
 ****************************************************************************/
bool PostMotor(ES_Event_t ThisEvent) {
    return ES_PostToService(MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunTemplateFSM

 Parameters
   ES_Event_t : the event to process

 Returns
   ES_Event_t, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 01/15/12, 15:23
 ****************************************************************************/
ES_Event_t RunMotor(ES_Event_t ThisEvent) {
    ES_Event_t ReturnEvent;
    ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

    switch (CurrentState) {
        case GameStart: // First Entering the Game
        {
            switch (ThisEvent.EventType) {
                case ES_INIT: //Sent by framework
                {
                    //confirm reached Init
                    printf("\rES_INIT received in MotorSM, %d\r\n", MyPriority);

                    //CW(); 
                    Forward_HalfSpeed();

//                    ES_Timer_InitTimer(PRINTSTALL_M1, 100);

                }
                    break;

                case ES_TIMEOUT: //for testing anti-stall
                {
                    //printf("\rRPM1%u\r\n",WheelRPM);
//                    ES_Timer_InitTimer(PRINTSTALL_M1, 100);

                }
                    break;

                case HW_BEACON:
                {
                    CurrentState = Loading;
                    
                    printf("\rHW_BEACON received in MotorSM, %d\r\n", MyPriority);
                    Stop();
                }
                    break;

                default:
                    ;
            }
        }
        break;
        case Loading: // First Entering the Game
        {
            switch (ThisEvent.EventType) {
                case ES_TIMEOUT:
                {
                    if (ThisEvent.EventParam == RELOAD_FWD_TIMER) {
                        Reverse_HalfSpeed();
                    }
                }
                break;
                case HW_LOADMODE: //Sent by framework
                {
                    printf("\rHW_LOADMODE received in MotorSM\r\n");

                    // Move forward until timeout or rear sensor far (currently set to rear sensor far)
                    Forward_HalfSpeed();
                    
                    // Currently commented out while we test using just rear sensor far instead
//                    ES_Timer_InitTimer(RELOAD_FWD_TIMER, RELOAD_FWD_DURATION);
                }
                    break;
                case HW_REARSENSORFAR:
                {
                    Reverse_HalfSpeed();
                    GoingBack = true;
                }
                    break;

//                case HW_REARSENSORNEAR: //slow down
//                {
//                    Reverse_HalfSpeed();
//                }
//                    break;
                    
                case HW_REARWALLCOLLIDE: //stop
                {
                    // avoid retriggering unless actively returning (i.e., rear limit switch could be pressed at start/from last time)
                    if (GoingBack) {
                        Stop();
                        GoingBack = false;
                        ResetRearFar();
                        ResetFrontNear();
                    }
                }
                    break;

                case HW_RISINGENCODER: //Sent by framework
                {
                    //DB_printf("\r1 %u\r\n", WheelRPM);
                }
                    break;

                case HW_RISINGENCODER2: //Sent by framework
                {
                    DB_printf("\r2 %u\r\n", WheelRPM2);

                }
                    break;
                case HW_DUNKMODE: //Sent by framework
                {
                    printf("\rHW_DUNKMODE received in MotorSM\r\n");

                    CurrentState = Dunking;
                    Forward_HalfSpeed();
                }
                    break;

                default:
                    ;
            }
        }
        break;
        case Dunking:
        {
            switch (ThisEvent.EventType) {
                case HW_FRONTWALLCOLLIDE: //stop
                {
                    CurrentState = Unloading;
                    Stop();
                    ES_Event_t TargetReached = {HW_TARGETREACHED, 0};
                    PostFollowerService(TargetReached);
                }
                break;
                case HW_MISALIGNED:
                {
                    CCW();
                }
                break;
                case HW_BEACON:
                {
                    Forward_HalfSpeed();
                }
                break;
                case HW_DUNKMODE:
                {
                    Forward_HalfSpeed();
                }
                break;
                case HW_LOADMODE:
                {
                    CurrentState = Loading;
                    Forward_HalfSpeed();
                }
                break;
                case HW_FRONTSENSORNEAR: //slow down
                {
//                    Forward_HalfSpeed();
                    printf("TARGET REACHED, NEED TO ADD POST STATEMENT\n");
                    // POST TARGET REACHED TO LEADER
                }
                break;
                case HW_REARSENSORNEAR: //slow down
                {
//                    Reverse_HalfSpeed();
                }
                break;                    
        }
        break;
        case Unloading:
        {
            // Critical: Don't accept any beacon align or misalign events while actively unloading
            switch (ThisEvent.EventType) {
                case HW_DUNKCOMPLETE:
                {
                    CurrentState = Returning;
                    Reverse_HalfSpeed();
                }
            break;
            }
        }
        break;
        case Returning:
        {
            switch (ThisEvent.EventType) {
                case HW_MISALIGNED:
                {
                    CCW();
                }
                break;
                case HW_BEACON:
                {
                    Reverse_HalfSpeed();
                }
                break;
            }
        }
        break;
        default:
            ;
        }
    } // end switch on Current State
    return ReturnEvent;
}

TemplateState_t QueryMotor(void) {
    return CurrentState;
}

/***************************************************************************
 private functions
 ***************************************************************************/
//Input Capture 1 ISR

void __ISR(_INPUT_CAPTURE_1_VECTOR, IPL7SOFT) IC1Handler(void) { //NOTE: Soft must match priority given
    __builtin_disable_interrupts();
    //restart anti-stall timer4
    TM4Start();
    
    //create static variable Direction to hold rise/fall (1/0), initialize to 0
    static uint16_t Direction = 1;

    if (Direction == 0) { //If Direction is 0 (second rise)        
        do {//do while (Input Capture Buffer NOT Empty Status)    

            //read IC1BUF into local variable TimeofLastFall.IC, cast it as uint16_t 
            KeepTime.ByBytes.IC = (uint16_t) IC1BUF;

            //If T2IF is pending and CapturedTime is after rollover (<0x8000) IFS0bits.T2IF is 1
            if (IFS0bits.T2IF == 1) {
                if (KeepTime.ByBytes.IC < 0x8000) {
                    //Increment the roll over counters
                    ++KeepTime.ByBytes.Rollover;
                    ++KeepTime2.ByBytes2.Rollover2;
                    ++FrontCounter.components.rollover;
                    ++RearCounter.components.rollover;
                    //clear the roll-over interrupt IFS0CLR w/ the Timer 3 mask _IFSF0_IC2IF_Mask
                    IFS0CLR = _IFS0_T2IF_MASK;
                }
            }

        } while (IC1CONbits.ICBNE != 0);
        
        //calculate new RPM measurement
        uint32_t encoderPulse = (KeepTime.Time - TimeofLastRise.Time);
        TimeofLastRise = KeepTime;

        //software filter of encoder noise
        if((encoderPulse != 0) && ((31250 / encoderPulse) < RPM_MAX_POSSIBLE)){
            WheelRPM = 31250 / encoderPulse;         
            //PostEvent  with parameter of the Current Time
            ES_Event_t EncoderEvent = {HW_RISINGENCODER, 0};
            PostMotor(EncoderEvent);
        }

        //set direction equal to count all encoder pulses
        Direction = 0;
        //clear the capture interrupt
        IFS0CLR = _IFS0_IC1IF_MASK;
    } else if (Direction == 1) { //If Direction is 1 (first rise)        
        do {//do while (Input Capture Buffer NOT Empty Status)     

            //read IC3BUF into local variable TimeofLastFall.IC, cast it as uint16_t
            KeepTime.ByBytes.IC = (uint16_t) IC1BUF;

            //If T2IF is pending and CapturedTime is after rollover (<0x8000) IFS0bits.T2IF is 1
            if (IFS0bits.T2IF == 1) {
                if (KeepTime.ByBytes.IC < 0x8000) {
                    //Increment the roll over counters
                    ++KeepTime.ByBytes.Rollover;
                    ++KeepTime2.ByBytes2.Rollover2;
                    ++FrontCounter.components.rollover;
                    ++RearCounter.components.rollover;
                    //clear the roll-over interrupt IFS0CLR w/ the Timer 3 mask _IFSF0_IC2IF_Mask
                    IFS0CLR = _IFS0_T2IF_MASK;
                }
            }

            TimeofLastRise.Time = 0;

            //restart anti-stall timer4
            TM4Start();

        } while (IC1CONbits.ICBNE != 0);

        //set direction equal to 0 to prep for next falling edge
        Direction = 0;
        //clear the capture interrupt
        IFS0CLR = _IFS0_IC1IF_MASK;
    }
    __builtin_enable_interrupts();
}

//Stall control interrupt for M1

void __ISR(_TIMER_4_VECTOR, IPL6SOFT) Timer4ISR(void) {
    //DB_printf("\rM1Stall\r\n");
    __builtin_disable_interrupts();
    IFS0CLR = _IFS0_T4IF_MASK;
    StallOutTimer = true;
    T4CONbits.ON = 0;
    WheelRPM = 0;
    __builtin_enable_interrupts();
}

//Input Capture ISR 

void __ISR(_INPUT_CAPTURE_2_VECTOR, IPL7SOFT) IC2Handler(void) { //NOTE: Soft must match priority given
    __builtin_disable_interrupts();
    //restart anti-stall timer5  
    TM5Start();


    //create static variable Direction to hold rise/fall (1/0), initialize to 0
    static uint16_t Direction = 1;

    if (Direction == 0) { //If Direction is 0 (second rise)        
        do {//do while (Input Capture Buffer NOT Empty Status)    

            //read IC1BUF into local variable TimeofLastFall.IC, cast it as uint16_t 
            KeepTime2.ByBytes2.IC2 = (uint16_t) IC2BUF;

            //If T2IF is pending and CapturedTime is after rollover (<0x8000) IFS0bits.T2IF is 1
            if (IFS0bits.T2IF == 1) {
                if (KeepTime2.ByBytes2.IC2 < 0x8000) {
                    //Increment the roll over counters
                    ++KeepTime.ByBytes.Rollover;
                    ++KeepTime2.ByBytes2.Rollover2;
                    ++FrontCounter.components.rollover;
                    ++RearCounter.components.rollover;
                    //clear the roll-over interrupt IFS0CLR w/ the Timer 3 mask _IFSF0_IC2IF_Mask
                    IFS0CLR = _IFS0_T2IF_MASK;
                }
            }

        } while (IC2CONbits.ICBNE != 0);

         //calculate new RPM measurement

        uint32_t encoderPulse2 = (KeepTime2.Time2 - TimeofLastRise2.Time2);
        TimeofLastRise2 = KeepTime2;

        //software filter of encoder noise
        if((encoderPulse2 != 0) && ((31250 / encoderPulse2) < RPM_MAX_POSSIBLE)){
            WheelRPM2 = 31250 / encoderPulse2;
            //PostEvent to print to serial
            ES_Event_t EncoderEvent2 = {HW_RISINGENCODER2, 0};
            PostMotor(EncoderEvent2);
        }
        
        //set direction equal to 0 to count all encoder counts
        Direction = 0;
        //clear the capture interrupt
        IFS0CLR = _IFS0_IC2IF_MASK;
    } else if (Direction == 1) { //If Direction is 1 (first rise)        
        do {//do while (Input Capture Buffer NOT Empty Status)     

            //read IC3BUF into local variable TimeofLastFall.IC, cast it as uint16_t
            KeepTime2.ByBytes2.IC2 = (uint16_t) IC2BUF;

            //If T2IF is pending and CapturedTime is after rollover (<0x8000) IFS0bits.T2IF is 1
            if (IFS0bits.T2IF == 1) {
                if (KeepTime2.ByBytes2.IC2 < 0x8000) {
                    //Increment the roll over counters
                    ++KeepTime.ByBytes.Rollover;
                    ++KeepTime2.ByBytes2.Rollover2;
                    ++FrontCounter.components.rollover;
                    ++RearCounter.components.rollover;
                    //clear the roll-over interrupt IFS0CLR w/ the Timer 3 mask _IFSF0_IC2IF_Mask
                    IFS0CLR = _IFS0_T2IF_MASK;
                }
            }

            TimeofLastRise2.Time2 = 0;

            //restart anti-stall timer5
            //TM5Start();  

        } while (IC2CONbits.ICBNE != 0);

        //set direction equal to 0 to prep for next falling edge
        Direction = 0;
        //clear the capture interrupt
        IFS0CLR = _IFS0_IC2IF_MASK;
    }
    __builtin_enable_interrupts();
}

//Stall control interrupt for M2

void __ISR(_TIMER_5_VECTOR, IPL6SOFT) Timer5ISR(void) {
    __builtin_disable_interrupts();
    IFS0CLR = _IFS0_T5IF_MASK;
    StallOutTimer2 = true;
    T5CONbits.ON = 0;
    WheelRPM2 = 0;
    //DB_printf("\rM2Stall\r\n");
    __builtin_enable_interrupts();    
}

/**ISR Timer2 Function**/
void __ISR(_TIMER_2_VECTOR, IPL6SOFT) Timer2Handler(void) {
    //Disable interrupts globally 
//    EnterCritical();
    __builtin_disable_interrupts();

    if (IFS0bits.T2IF == 1) {//If T3IF is pending (IC ISR has NOT fired and handled already)
        //Increment roll-over counter
        ++KeepTime.ByBytes.Rollover;
        ++KeepTime2.ByBytes2.Rollover2;
        ++FrontCounter.components.rollover;
        ++RearCounter.components.rollover;
        //clear roll-over interrupt IEC0bits.T
        IFS0CLR = _IFS0_T2IF_MASK;
    }
    //Enable interrupts 
//    ExitCritical();
    __builtin_enable_interrupts();
}

//Closed Loop control in Timer1

void __ISR(_TIMER_1_VECTOR, IPL6SOFT) ClosedLoopHandler(void) {
    __builtin_disable_interrupts();
    //error has to be SIGNED
    //error from last measured velocity to desired velocity
    int16_t error = DesiredVel - WheelRPM;
    int16_t error2 = DesiredVel2 - WheelRPM2;

    ErrorSum = ErrorSum + error;
    ErrorSum2 = ErrorSum2 + error2;

    int16_t newDC2 = (1.68 * (error2 + (ErrorSum2 * 0.0833) + 3 * (error2 - ErrorLast2)) / 100) * PULSEWIDTH; //PID
    int16_t newDC = (1.68 * (error + (ErrorSum * 0.08333) + 3 * (error - ErrorLast)) / 100) * PULSEWIDTH; //PID

    //clear the capture interrupt
    IFS0CLR = _IFS0_T1IF_MASK;
    __builtin_enable_interrupts();
}

static void TM3OCInit() {
    //sets up timer 3 for output capture
    T3CONbits.ON = 0;
    //clear the TCS control bit (T3CON<1>) to select internal PBCLK clock source
    T3CONbits.TCS = 0;
    //select pre-scale 1
    //note that until the time constant of the motor is known, this will be set to PBClck
    //prescalar 111/256 110/64 101/32 100/16 011/8 010/4 001/2 000/1 
    T3CONbits.TCKPS = 0b001;
    //load the period register PR3
    //note that to determine time constant of the motor this will be set for approximately 1 ms period
    //and then will be adjusted when time constant is known
    PR3 = (uint16_t) PULSEWIDTH;
    //clear the T3IF interrupt flag in the IFS0 register (write to IFS0CLR)
    IFS0CLR = _IFS0_T3IF_MASK;
    //clear timer register TMR3
    TMR3 = 0;
    //sets up output capture

    //setting up Motor 1 1/2 
    //turn off the OC1
    OC1CONbits.ON = 0;
    //select timer 3
    OC1CONbits.OCTSEL = 1;
    //set PWM mode with no fault detect
    OC1CONbits.OCM = 0b110;
    //set DC to initial value
    //in the OCR register
    OC1R = (uint16_t) DC_INITIAL;
    //in the OCRS register
    OC1RS = (uint16_t) DC_INITIAL;
    //map OC1 pin
    RPB15R = 0b0101; //OC1 Table 11-2 Output Pin Selection
    //debugging RPB4R = 0b0101; //OC1 Table 11-2 Output Pin Selection
    //Turn OC back on
    OC1CONbits.ON = 1;

    //setting up Motor 1 2/2 
    //turn off the OC2
    OC2CONbits.ON = 0;
    //select timer 3
    OC2CONbits.OCTSEL = 1;
    //set PWM mode with no fault detect
    OC2CONbits.OCM = 0b110;
    //set DC to initial value
    //in the OCR register
    OC2R = (uint16_t) DC_INITIAL_2;
    //in the OCRS register
    OC2RS = (uint16_t) DC_INITIAL_2;
    //map OC1 pin
    RPA1R = 0b101; //OC2 Table 11-2 Output Pin Selection
    //Turn OC back on
    OC2CONbits.ON = 1;

    //setting up Motor 2 1/2 
    //turn off the OC3
    OC3CONbits.ON = 0;
    //select timer 3
    OC3CONbits.OCTSEL = 1;
    //set PWM mode with no fault detect
    OC3CONbits.OCM = 0b110;
    //set DC to initial value
    //in the OCR register
    OC3R = (uint16_t) DC_INITIAL_3;
    //in the OCRS register
    OC3RS = (uint16_t) DC_INITIAL_3;
    //map OC1 pin
    RPB10R = 0b101; //OC3 Table 11-2 Output Pin Selection
    //Turn OC back on
    OC3CONbits.ON = 1;

    //setting up Motor 2 2/2 
    //turn off the OC4
    OC4CONbits.ON = 0;
    //select timer 3
    OC4CONbits.OCTSEL = 1;
    //set PWM mode with no fault detect
    OC4CONbits.OCM = 0b110;
    //set DC to initial value
    //in the OCR register
    OC4R = (uint16_t) DC_INITIAL_4;
    //in the OCRS register
    OC4RS = (uint16_t) DC_INITIAL_4;
    //map OC1 pin
    RPB2R = 0b101; //OC4 Table 11-2 Output Pin Selection
    //Turn OC back on
    OC4CONbits.ON = 1;

    //Turn timer 3 back on
    T3CONbits.ON = 1;

    return;
}

static void TM2ICInit() {

    //sets up timer 2 for input capture
    //clear the ON control bits (T3CON<15>) to disable timer
    T2CONbits.ON = 0;
    //clear the TCS control bit (T3CON<1>) to select internal PBCLK clock source
    T2CONbits.TCS = 0;
    //prescale will depend on final encoder shaft velocities
    //reduce prescalar 111/256 110/64 101/32 100/16 011/8 010/4 001/2 000/1
    T2CONbits.TCKPS = 0b110;
    //load the period register PR3
    //period registor will be full buffer to reduce roll over
    PR2 = 0xFFFF;
    //clear T2IF interrupt flag
    //clear the T3IF interrupt flag in the IFS0 register (write to IFS0CLR)
    IFS0CLR = _IFS0_T2IF_MASK;
    //timer interrupts needed to count TMR3 rollover
    //configure interrupt priority and subpriority in the IPC3 register (IPC3bits.T3IP = 6)
    IPC2bits.T2IP = 6;
    //set the T3IE interrupt enable bit in the IEC0 register (IEC0SET = _IEC0_T3IE_MASK)
    IEC0SET = _IEC0_T2IE_MASK;
    //clear timer register TMR2
    TMR2 = 0;
    //set the ON control bit to enable timer
    T2CONbits.ON = 1;

    //sets up input capture
    //disable IC
    IC1CONbits.ON = 0;
    //set continue to operate in idle mode
    IC1CONbits.SIDL = 0;
    //set First Capture Edge Select (FEDGE) bit to capture rise edge first (1), so can keep track of bits
    IC1CONbits.FEDGE = 1;
    //set 16-bit timer resource (C32 = 0)
    IC1CONbits.C32 = 0;
    //select timer 2 with ICTMR = 1
    IC1CONbits.ICTMR = 1;
    //set to interrupt on every capture event (ICI = 0)
    IC1CONbits.ICI = 0;
    //set input capture mode to ONLY rising edges
    IC1CONbits.ICM = 0b011;
    //set interrupt priority and sub-priority level (IPC3bits.IC3IP = 7)
    IPC1bits.IC1IP = 7;
    //clear any pending masks
    IFS0CLR = _IFS0_IC1IF_MASK;
    //enable interrupt
    IEC0SET = _IEC0_IC1IE_MASK;
    //re-enable IC
    IC1CONbits.ON = 1;

    //setup for reading in encoder values for IC1   
    //clear the ANSEL bit to disable analog on the pin **the Morse signal has 
    ANSELA &= ~((uint16_t) (1 << 4));
    //configure the specified pin as Digital Input using TRISx
    // If a data direction bit is 1, the corresponding I/O port pin is an input
    TRISA |= ((uint16_t) (1 << 4));
    //map the pin RA4 to IC1R
    IC1R = 0b0010; //0010 for RA4

    //sets up input capture
    //disable IC
    IC2CONbits.ON = 0;
    //set continue to operate in idle mode
    IC2CONbits.SIDL = 0;
    //set First Capture Edge Select (FEDGE) bit to capture rise edge first (1), so can keep track of bits
    IC2CONbits.FEDGE = 1;
    //set 16-bit timer resource (C32 = 0)
    IC2CONbits.C32 = 0;
    //select timer 2 with ICTMR = 1
    IC2CONbits.ICTMR = 1;
    //set to interrupt on every capture event (ICI = 0)
    IC2CONbits.ICI = 0;
    //set input capture mode to ONLY rising edges
    IC2CONbits.ICM = 0b011;
    //set interrupt priority and sub-priority level (IPC3bits.IC3IP = 7)
    IPC2bits.IC2IP = 7;
    //clear any pending masks
    IFS0CLR = _IFS0_IC2IF_MASK;
    //enable interrupt
    IEC0SET = _IEC0_IC2IE_MASK;
    //re-enable IC
    IC2CONbits.ON = 1;

    //setup for reading in encoder values for IC1   
    //clear the ANSEL bit to disable analog on the pin **the Morse signal has 
    ANSELB &= ~((uint16_t) (1 << 14));
    //configure the specified pin as Digital Input using TRISx
    // If a data direction bit is 1, the corresponding I/O port pin is an input
    TRISB |= ((uint16_t) (1 << 14));
    //map the pin RA3 to IC2R
    IC2R = 0b0000; //0001 for RA3

    return;
}

static void TM1Init() {

    //sets up timer 1 for closed loop control
    //clear the ON control bits (T3CON<15>) to disable timer
    T1CONbits.ON = 0;
    //clear the TCS control bit (T3CON<1>) to select internal PBCLK clock source
    T1CONbits.TCS = 0;
    T1CONbits.TCKPS = 0b11; //11/256 10/64 01/8 00/1
    //load the period register PR1, divisor of 256 and PR 75 gives 0.99mS period
    PR1 = 75;
    //clear T1IF interrupt flag
    //clear the T3IF interrupt flag in the IFS0 register (write to IFS0CLR)
    IFS0CLR = _IFS0_T1IF_MASK;
    //configure interrupt priority
    IPC1bits.T1IP = 6;
    //NOTE: this is ONLY necessary if the IC bits roll-over 
    //set T2 priority to 6
    //enable interrupt
    IEC0SET = _IEC0_T1IE_MASK;
    //clear timer register TMR2
    TMR1 = 0;
    //set the ON control bit to enable timer
    T1CONbits.ON = 1;

    return;
}

//stall control for M1

static void TM4InitOnly(void) //Code adapted from Ed's TestHarnessService0
{
    // turn timer off
    T4CONbits.ON = 0;
    // Use internal peripheral clock
    T4CONbits.TCS = 0;
    // set prescale to 1:256 (IC is set to 1:64)
    T4CONbits.TCKPS = 0b11;
    // load period value
    //if reached the full value of this clock, then IC has not been triggered 
    //4X its full length
    PR4 = 0xFFFF;
    // set priority
    IPC4bits.T4IP = 6;
    // clear interrupt flag
    IFS0bits.T4IF = 0;
    // disable the timer interrupt
    IEC0bits.T4IE = 1;

    return;
}

//Start timer 4, Stall control for M1

static void TM4Start(void) //Code adapted from Ed's TestHarnessService0
{
    StallOutTimer = false;
    // clear timer
    TMR4 = 0;
    // start timer
    T4CONbits.ON = 1;

    return;
}

//stall control for M2

static void TM5InitOnly(void) //Code adapted from Ed's TestHarnessService0
{
    // turn timer off
    T5CONbits.ON = 0;
    // Use internal peripheral clock
    T5CONbits.TCS = 0;
    // set prescale to 1:256 (IC is set to 1:64)
    T5CONbits.TCKPS = 0b11;
    // load period value
    //if reached the full value of this clock, then IC has not been triggered 
    //4X its full length
    PR5 = 0xFFFF;
    // set priority
    IPC5bits.T5IP = 6;
    // clear interrupt flag
    IFS0bits.T5IF = 0;
    // disable the timer interrupt
    IEC0bits.T5IE = 1;

    return;
}

//Start timer 5, Stall control for M2

static void TM5Start(void) //Code adapted from Ed's TestHarnessService0
{
    StallOutTimer2 = false;
    T5CONbits.ON = 0;
    // clear timer
    TMR5 = 0;
    // start timer
    T5CONbits.ON = 1;

    return;
}

static void ForwardM1(float requestedDC) {
    OC1RS = (uint16_t) 0;
    OC2RS = (uint16_t) PULSEWIDTH * requestedDC;
    return;
}

static void ReverseM1(float requestedDC) {
    OC1RS = (uint16_t) PULSEWIDTH * requestedDC;
    OC2RS = (uint16_t) 0;
    return;
}

static void ForwardM2(float requestedDC) {
    OC3RS = (uint16_t) PULSEWIDTH * requestedDC;
    OC4RS = (uint16_t) 0;
    return;
}

static void ReverseM2(float requestedDC) {
    OC3RS = (uint16_t) 0;
    OC4RS = (uint16_t) PULSEWIDTH * requestedDC;
    return;
}

static void Forward_FullSpeed() {
    ForwardM1(DC_FULLSPEED);
    ForwardM2(MOTOR2_TO_MOTOR1_RATIO * DC_FULLSPEED);
}

static void Forward_HalfSpeed() {
    ForwardM1(DC_HALFSPEED);
    ForwardM2(MOTOR2_TO_MOTOR1_RATIO * DC_HALFSPEED);
}

static void Reverse_FullSpeed() {
    ReverseM1(DC_FULLSPEED);
    ReverseM2(MOTOR2_TO_MOTOR1_RATIO * DC_FULLSPEED);
}

static void Reverse_HalfSpeed() {
    ReverseM1(DC_HALFSPEED);
    ReverseM2(MOTOR2_TO_MOTOR1_RATIO * DC_HALFSPEED);
}

static void CW() {
    ForwardM1(DC_TURN);
    ReverseM2(MOTOR2_TO_MOTOR1_RATIO * DC_TURN);
}

static void CCW() {
    ReverseM1(DC_TURN);
    ForwardM2(MOTOR2_TO_MOTOR1_RATIO * DC_TURN);
}

static void Stop() {
    ForwardM1(0);
    ForwardM2(0);
}

//set desired velocity
//reset error sum to 0 every time set new desired velocity
//set the direction the robot

static void CL_Forward(uint8_t requestedRPM){
	DesiredVel = requestedRPM;
	ErrorSum = 0;
	ErrorLast = 0;
	Direction = 1;
    
	DesiredVel2 = requestedRPM;
	ErrorSum2 = 0;
	ErrorLast2 = 0;
	Direction2 = 1;
}

static void CL_Reverse(uint8_t requestedRPM){
	DesiredVel = requestedRPM;
	ErrorSum = 0;
	ErrorLast = 0;
	Direction = 0;
    
	DesiredVel2 = requestedRPM;
	ErrorSum2 = 0;
	ErrorLast2 = 0;
	Direction2 = 0;
}

static void CL_CW(uint8_t requestedRPM){
	DesiredVel = requestedRPM;
	ErrorSum = 0;
	ErrorLast = 0;
	Direction = 1;
    
	DesiredVel2 = requestedRPM;
	ErrorSum2 = 0;
	ErrorLast2 = 0;
	Direction2 = 0;
}

static void CL_CCW(uint8_t requestedRPM){
	DesiredVel = requestedRPM;
	ErrorSum = 0;
	ErrorLast = 0;
	Direction = 0;
    
	DesiredVel2 = requestedRPM;
	ErrorSum2 = 0;
	ErrorLast2 = 0;
	Direction2 = 1;
}
