/****************************************************************************
 Module
   UltrasonicService.c
/*----------------------------- Include Files -----------------------------*/
// This module
#include "UltrasonicService.h"
#include "MotorSM.h"

// debugging printf()
//#include "dbprintf.h"

#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_DeferRecall.h"
#include "sys/attribs.h"
#include "PIC32PortHAL.h"
#include "MotorSM.h"
#include "FollowerService.h"

/*----------------------------- Module Defines ----------------------------*/
#define ONE_SEC 1000
#define HALF_SEC 500
#define THIRD_SEC 300
#define MILLI_SEC 1

#define CLK_HZ 20000000
#define PRESCALE 64

#define CM_FACTOR 58
#define IN_FACTOR 148
#define US_IN_S 1000000

#define SENSOR_THRESHOLD 50 // in cm 

#define ENTER_POST     ((MyPriority<<3)|0)
#define ENTER_RUN      ((MyPriority<<3)|1)
#define ENTER_TIMEOUT  ((MyPriority<<3)|2)

#define TEST_INT_POST
/*---------------------------- Module Functions ---------------------------*/
static void InitUltrasonic(void);
static void InitTMR2(void);
static void InitIC3(void);
static void InitIC4(void);
/*---------------------------- Module Variables ---------------------------*/
// with the introduction of Gen2, we need a module level Priority variable
static uint8_t MyPriority;
// add a deferral queue for up to 3 pending deferrals +1 to allow for overhead
static ES_Event_t DeferralQueue[3 + 1];

static uint32_t LastFrontTime, LastRearTime, FrontPulseWidth, RearPulseWidth;
static bool IsFrontRisingEdge, IsRearRisingEdge, UltrasonicReady;
static bool HasSentRearFarEvent, HasSentFrontNearEvent;

volatile counter FrontCounter;
volatile counter RearCounter;
/*------------------------------ Module Code ------------------------------*/
bool InitUltrasonicService(uint8_t Priority) {
    ES_Event_t ThisEvent;

    MyPriority = Priority;

    IsFrontRisingEdge = true;
    IsRearRisingEdge = true;
    
    HasSentRearFarEvent = false;
    HasSentFrontNearEvent = false;
    
    InitUltrasonic();
    InitIC3();
    InitIC4();

    // post the initial transition event
    ThisEvent.EventType = ES_INIT;
    if (ES_PostToService(MyPriority, ThisEvent) == true) {
        return true;
    } else {
        return false;
    }
}

bool PostUltrasonicService(ES_Event_t ThisEvent) {
    return ES_PostToService(MyPriority, ThisEvent);
}

ES_Event_t RunUltrasonicService(ES_Event_t ThisEvent) {
    ES_Event_t ReturnEvent;
    ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

    switch (ThisEvent.EventType) {
        case ES_INIT:
        {
            ES_Timer_InitTimer(ULTRASONIC_TIMER, THIRD_SEC);
        }
        break;
        case ES_TIMEOUT: // re-start timer & announce
        {
            UltrasonicReady = true;
            LATBCLR = 1 << 9;
            LATBCLR = 1 << 4;
            
            ES_Timer_InitTimer(ULTRASONIC_TIMER, THIRD_SEC);
        }
        break;
        case EV_ECHO:
        {
            float PulseWidth = (float) ThisEvent.EventParam * PRESCALE * US_IN_S / CLK_HZ / CM_FACTOR;
            printf("Front Ultrasonic Distance (cm): %f\r\n", PulseWidth);
            ES_Event_t FrontSensorNearEvent = {HW_FRONTSENSORNEAR, PulseWidth};
            if(PulseWidth < SENSOR_THRESHOLD && !HasSentFrontNearEvent){
                printf("Sending front sensor near event to motor with pulse width: %d\r\n", PulseWidth);
                HasSentFrontNearEvent = true;
                PostMotor(FrontSensorNearEvent);
            }
        }
        break;
        case EV_REAR_ECHO:
        {
            float PulseWidth = (float) ThisEvent.EventParam * PRESCALE * US_IN_S / CLK_HZ / CM_FACTOR;
             printf("Rear Ultrasonic Distance (cm): %f\r\n", PulseWidth);
            if(PulseWidth < SENSOR_THRESHOLD && !HasSentRearFarEvent){
                ES_Event_t RearSensorNearEvent = {HW_REARSENSORFAR, PulseWidth};
                printf("Sending rear sensor near event to motor with pulse width: %d\r\n", PulseWidth);
                HasSentRearFarEvent = true;
                PostMotor(RearSensorNearEvent);
            }
        }
        break;
        default:
        {
        }
            break;
    }

    return ReturnEvent;
}
/***************************************************************************
 private functions
 ***************************************************************************/

void ResetFrontNear(void) {
    HasSentFrontNearEvent = false;
}
void ResetRearFar(void) {
    HasSentRearFarEvent = false;
}

void InitUltrasonic(void) {
    UltrasonicReady = false;
    
    // front sensor = IC3
    PortSetup_ConfigureDigitalInputs(_Port_B, _Pin_5); // echo is input
    PortSetup_ConfigureDigitalOutputs(_Port_B, _Pin_9); // trigger is output
    LATBSET = 1 << 9;
    
    // rear sensor = IC4
    PortSetup_ConfigureDigitalInputs(_Port_B, _Pin_3); // rear echo RB3 is input
    PortSetup_ConfigureDigitalOutputs(_Port_B, _Pin_4); // rear trigger RB4 is output
    LATBSET = 1 << 4;
    
}

void InitIC3(void) {
    IC3CONbits.ON = 0;

    IC3BUF = 0;
    IC3CONbits.FEDGE = 1; // rising edge
    IC3CONbits.C32 = 0;
    IC3CONbits.ICTMR = 1;
    IC3CONbits.ICI = 0;
    IC3CONbits.ICOV = 0;
    IC3CONbits.ICBNE = 0;
    IC3CONbits.SIDL = 0;
    IC3CONbits.ICM = 0b110; // every edge starting with FEDGE

    IC3R = 0b0001; // RPB5
    IEC0SET = _IEC0_IC3IE_MASK;
    IFS0CLR = _IFS0_IC3IF_MASK;
    INTCONbits.MVEC = 1;
    IPC3bits.IC3IP = 7;

    IC3CONbits.ON = 1;
}

void InitIC4(void){
    IC4CONbits.ON = 0;

    IC4BUF = 0;
    IC4CONbits.FEDGE = 1; // rising edge
    IC4CONbits.C32 = 0;
    IC4CONbits.ICTMR = 1;
    IC4CONbits.ICI = 0;
    IC4CONbits.ICOV = 0;
    IC4CONbits.ICBNE = 0;
    IC4CONbits.SIDL = 0;
    IC4CONbits.ICM = 0b110; // every edge starting with FEDGE

    IC4R = 0b0001; // RPB3
    IEC0SET = _IEC0_IC4IE_MASK;
    IFS0CLR = _IFS0_IC4IF_MASK;
    INTCONbits.MVEC = 1;
    IPC4bits.IC4IP = 7;

    IC4CONbits.ON = 1;
}

/***************************************************************************
 interrupt service routines
 ***************************************************************************/

void __ISR(_INPUT_CAPTURE_3_VECTOR, IPL7SOFT) _HW_IC3Handler(void) {
    __builtin_disable_interrupts();

    do {
        LATBSET = 1 << 9;
        FrontCounter.components.buf = (uint16_t) IC3BUF;
        if (IFS0bits.T2IF && FrontCounter.components.buf < 0x8000) {
            ++KeepTime.ByBytes.Rollover;
            ++KeepTime2.ByBytes2.Rollover2;
            ++FrontCounter.components.rollover;  
            ++RearCounter.components.rollover;
            IFS0CLR = _IFS0_T2IF_MASK;
        }

        FrontPulseWidth = FrontCounter.all - LastFrontTime;
        LastFrontTime = FrontCounter.all;
        if (!IsFrontRisingEdge) {
            ES_Event_t EchoEvent;
            EchoEvent.EventParam = FrontPulseWidth;
            EchoEvent.EventType = EV_ECHO;
            if (UltrasonicReady) {
                PostUltrasonicService(EchoEvent);
            }
        }
        
        IFS0CLR = _IFS0_IC3IF_MASK;

        IsFrontRisingEdge = !IsFrontRisingEdge;
        
    } while (IC3CONbits.ICBNE != 0);

    IFS0CLR = _IFS0_IC3IF_MASK;

    __builtin_enable_interrupts();
}

void __ISR(_INPUT_CAPTURE_4_VECTOR, IPL7SOFT) _HW_IC4Handler(void) {
    __builtin_disable_interrupts();

    do {
        LATBSET = 1 << 4;
        RearCounter.components.buf = (uint16_t) IC4BUF;
        if (IFS0bits.T2IF && RearCounter.components.buf < 0x8000) {
            ++KeepTime.ByBytes.Rollover;
            ++KeepTime2.ByBytes2.Rollover2;
            ++FrontCounter.components.rollover;
            ++RearCounter.components.rollover;
            IFS0CLR = _IFS0_T2IF_MASK;
        }

        RearPulseWidth = RearCounter.all - LastRearTime;
        LastRearTime = RearCounter.all;
        if (!IsRearRisingEdge) {
            ES_Event_t EchoEvent;
            EchoEvent.EventParam = RearPulseWidth;
            EchoEvent.EventType = EV_REAR_ECHO;
            if (UltrasonicReady) {
                PostUltrasonicService(EchoEvent);
            }
        }
        
        IFS0CLR = _IFS0_IC4IF_MASK;

        IsRearRisingEdge = !IsRearRisingEdge;
        
    } while (IC4CONbits.ICBNE != 0);

    IFS0CLR = _IFS0_IC4IF_MASK;

    __builtin_enable_interrupts();
}